﻿//////////////////////////////////////////////
// DofComplexEffect.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkAstraeus
{
	class BlurComplexEffect ;
	class TextureCopyEffect ;
}

namespace nkGraphics
{
	class Camera ;
	class Program ;
	class Sampler ;
	class Shader ;
	class TargetOperations ;
	class Texture ;
	class Viewport ;
}

/// Includes ---------------------------------

// nkAstraeus
#include "../../../Dll/DllDefines.h"

#include "../ComplexEffect.h"

#include "DofComplexEffectInfo.h"

/// Class ------------------------------------

namespace nkAstraeus
{
	class DLL_ASTRAEUS_EXPORT DofComplexEffect final : public ComplexEffect
	{
		private :

			// Attributes
			// Program
			DofComplexEffectInfo _loadedInfo ;
			nkGraphics::CompositorNode* _loadedNode ;

			// Info
			DofComplexEffectInfo _info ;

			// Resources
			nkGraphics::Shader* _downSamplingShader ;
			BlurComplexEffect* _blurringEffectNear ;
			BlurComplexEffect* _blurringEffectFar ;
			BlurComplexEffect* _blurringEffectFadeInfo ;
			nkGraphics::Shader* _blendShader ;
			nkGraphics::Texture* _downSampledTextureNear ;
			nkGraphics::Texture* _downSampledTextureFar ;
			nkGraphics::Texture* _downSampledTextureFadeInfo ;
			nkGraphics::Texture* _outputTexture ;
			nkGraphics::Sampler* _sampler ;
			nkGraphics::Sampler* _samplerDepth ;
			nkGraphics::Viewport* _viewportDown ;
			nkGraphics::Viewport* _viewportHigh ;

			nkGraphics::TargetOperations* _downSamplingOp ;
			nkGraphics::TargetOperations* _blendOp ;

		private :

			// Functions
			// Textures
			bool _prepareTextures () ;
			bool _prepareSamplers () ;
			// Viewports
			bool _prepareViewports () ;
			// Shaders
			bool _prepareShaders () ;

			// Compositor
			bool _prepareCompositorNode () ;

		public :

			// Constructor, destructor
			DofComplexEffect (nkMemory::StringView name) ;
			~DofComplexEffect () ;

			// Getters
			virtual COMPLEX_EFFECT_TYPE getType () const override ;
			nkGraphics::Texture* getSourceTexture () const ;
			nkGraphics::Texture* getSourceDepth () const ;
			nkGraphics::Texture* getOutputTexture () const ;
			nkGraphics::Camera* getCamera () const ;
			float getNearPlaneDistance () const ;
			float getFocalPlaneDistance () const ;
			float getFocalPlaneThickness () const ;
			float getFarPlaneDistance () const ;
			unsigned int getBlurStrength () const ;
			bool getLinearDepth () const ;

			// Setters
			void setSourceTexture (nkGraphics::Texture* value) ;
			void setSourceDepth (nkGraphics::Texture* value) ;
			void setCamera (nkGraphics::Camera* value) ;
			void setNearPlaneDistance (float value) ;
			void setFocalPlaneDistance (float value) ;
			void setFocalPlaneThickness (float value) ;
			void setFarPlaneDistance (float value) ;
			void setBlurStrength (unsigned int value) ;
			void setLinearDepth (bool value) ;

			// Load
			virtual bool load () override ;
			virtual void unload () override ;
	} ;
}