﻿//////////////////////////////////////////////
// TextureManager.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class Texture ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// nkCommon
#include <nilkinsCommon/Patterns/SingletonClass.h>

// nkMemory
#include <NilkinsMemory/Containers/StringView.h>

// Standards
#include <mutex>
#include <string>
#include <string_view>
#include <unordered_map>

/// Class ------------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT TextureManager final : public nkCommon::SingletonClass<TextureManager>
	{
		friend class nkCommon::SingletonClass<TextureManager> ;

		private :

			// Constructeur
			TextureManager () ;
		
			// Attributs
			std::unordered_map<std::string, Texture*> _memory ;
				
			// Active render targets
			Texture* _activeRenderTarget ;
			Texture* _activeDepthTarget ;

			// Threading
			std::mutex _mutex ;

		public :
		
			// Constructeur, destructeur
			~TextureManager () ;

			// Manipulation mémoire
			Texture* createOrRetrieve (const nkMemory::StringView& name) ;
			Texture* get (const nkMemory::StringView& name) ;
			Texture* getByIndex (unsigned int index) ;
			void rename (const nkMemory::StringView& currentName, const nkMemory::StringView& newName) ;
			void erase (const nkMemory::StringView& name) ;

			// Tex actives
			void setActiveRenderTarget (Texture* target) ;
			void setActiveDepthTarget (Texture* target) ;
			Texture* getActiveRenderTarget () const ;
			Texture* getDepthTarget () const ;
	} ;
}