﻿//////////////////////////////////////////////
// ComponentManager.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkWinUi
{
	class Component ;
	class MainSystem ;
}

/// Includes ---------------------------------

// nkWinUi
#include "../Dll/DllDefines.h"

#include "ComponentType.h"

// nkCommon
#include <nilkinsCommon/Patterns/MaybeSingletonClass.h>

// nkMemory
#include <NilkinsMemory/Containers/StringView.h>

// Standards
#include <unordered_map>
#include <string>

/// Class ------------------------------------

namespace nkWinUi
{
	class DLL_WINUI_EXPORT ComponentManager final : public nkCommon::MaybeSingletonClass<ComponentManager>
	{
		friend class nkCommon::MaybeSingletonClass<ComponentManager> ;
		friend class MainSystem ;

		private :

			// Attributes
			std::unordered_map<std::string, Component*> _components ;
			MainSystem* _parentSystem ;

		private :

			// Constructor, destructor
			ComponentManager () ;
			ComponentManager (MainSystem* parentSystem) ;
			~ComponentManager () ;
		
		public :

			// Manipulations
			Component* createOrRetrieve (const nkMemory::StringView& name, COMPONENT_TYPE componentType) ;
			Component* get (const nkMemory::StringView& name) const ;
			Component* getByIndex (unsigned int index) const ;
			void erase (const nkMemory::StringView& name) ;
	} ;
}