﻿//////////////////////////////////////////////
// TabWindow.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkWinUi
{
	class Window ;
}

/// Includes ---------------------------------

// nkWinUi
#include "Component.h"

#include "../Dll/DllDefines.h"

// Standards
#include <string>
#include <unordered_map>

/// Class ------------------------------------
	
namespace nkWinUi
{
	class DLL_WINUI_EXPORT TabWindow : public Component
	{
		public :

			// Tab class to keep needed info
			class DLL_WINUI_EXPORT Tab
			{
				protected :
						
					// Le titre
					std::string _label ;
					// La Window associee
					Window* _associatedWin ;
					// Son id
					int _id ;

					// La tab window parente
					TabWindow* _parentWindow ;

				public :

					// Constructeur, destructeur
					Tab (TabWindow* parentWindow) ;
					virtual ~Tab () ;

					// Getters
					nkMemory::StringView getLabel () const ;
					Window* getAssociatedWin () const ;
					int getId () const ;
					TabWindow* getParentWindow () const ;

					// Setters
					virtual void setLabel (const nkMemory::StringView& label, bool autoUpdate = true) ;
					void setAssociatedWin (Window* win) ;
					void setId (int id) ;
			} ;

		protected :

			// Functions
			virtual Tab* _allocateTab (TabWindow* parentWindow) const = 0 ;

		protected :
		
			// Attributes
			// Tabs, also associated to an id for identifying
			std::unordered_map<int, Tab*> _indicesMemory ;
			std::vector<Tab*> _tabs ;

			// Active tab
			Tab* _activeTab ;

			// Callback when changing tab
			std::function<bool (TabWindow*, TabWindow::Tab*, bool)> _tabSwitchCallback ;
		
		public :
		
			// Constructor, destructor
			TabWindow (MainSystem* parentSystem) ;
			virtual ~TabWindow () ;

			// For tabs
			virtual Tab* addTab (const nkMemory::StringView& label) ;
			Tab* getTabByLabel (const nkMemory::StringView& label) const ;
			Tab* getTabById (int id) const ;
			Tab* getTabByIndex (unsigned int index) const ;
			virtual Tab* getTabByMouseRel (const nkMaths::Point& mouseRel) const = 0 ;
			int getTabIndex (Tab* tab) const ;
			unsigned int getTabCount () const ;
			virtual void reorderTab (Tab* tab, unsigned int newIndex) ;
			virtual void eraseTab (Tab* tab) ;

			// Concerning layouts
			virtual Layout* setLayout (LAYOUT_TYPE layoutType) override ;

			// Active tab
			virtual void setActiveTab (Tab* toActivate) ;
			Tab* getActiveTab () const ;

			// Sizing inside
			virtual void adaptToDisplayArea (Window* toAdapt) const = 0 ;

			// Callbacks
			void setTabSwitchCallback (std::function<bool (TabWindow*, TabWindow::Tab*, bool)> value) ;
			const std::function<bool (TabWindow*, TabWindow::Tab*, bool)>& getTabSwitchCallback () const ;

			// Import / Export
			virtual void exportIntrospection (nkExport::Node* rootNode) override ;
			void importClassFromTree (nkExport::Node* rootNode) override ;
	} ;
}